@@ -5,7 +5,7 @@ use std::ptr;
 use glib::{prelude::*, translate::*};
 use gst::subclass::prelude::*;
 
-use crate::{Aggregator, AggregatorPad};
+use crate::{ffi, Aggregator, AggregatorPad};
 
 pub trait AggregatorImpl: AggregatorImplExt + ElementImpl {
     fn flush(&self) -> Result<gst::FlowSuccess, gst::FlowError> {
@@ -16,8 +16,8 @@ pub trait AggregatorImpl: AggregatorImplExt + ElementImpl {
         self.parent_clip(aggregator_pad, buffer)
     }
 
-    #[cfg(any(feature = "v1_18", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
+    #[cfg(feature = "v1_18")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_18")))]
     fn finish_buffer_list(
         &self,
         buffer_list: gst::BufferList,
@@ -33,8 +33,8 @@ pub trait AggregatorImpl: AggregatorImplExt + ElementImpl {
         self.parent_sink_event(aggregator_pad, event)
     }
 
-    #[cfg(any(feature = "v1_18", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
+    #[cfg(feature = "v1_18")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_18")))]
     fn sink_event_pre_queue(
         &self,
         aggregator_pad: &AggregatorPad,
@@ -47,8 +47,8 @@ pub trait AggregatorImpl: AggregatorImplExt + ElementImpl {
         self.parent_sink_query(aggregator_pad, query)
     }
 
-    #[cfg(any(feature = "v1_18", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
+    #[cfg(feature = "v1_18")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_18")))]
     fn sink_query_pre_queue(
         &self,
         aggregator_pad: &AggregatorPad,
@@ -122,111 +122,25 @@ pub trait AggregatorImpl: AggregatorImplExt + ElementImpl {
         self.parent_decide_allocation(query)
     }
 
-    #[cfg(any(feature = "v1_18", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
+    #[cfg(feature = "v1_18")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_18")))]
     fn negotiate(&self) -> bool {
         self.parent_negotiate()
     }
 
-    #[cfg(any(feature = "v1_18", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
+    #[cfg(feature = "v1_18")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_18")))]
     fn peek_next_sample(&self, pad: &AggregatorPad) -> Option<gst::Sample> {
         self.parent_peek_next_sample(pad)
     }
 }
 
-pub trait AggregatorImplExt: ObjectSubclass {
-    fn parent_flush(&self) -> Result<gst::FlowSuccess, gst::FlowError>;
-
-    fn parent_clip(
-        &self,
-        aggregator_pad: &AggregatorPad,
-        buffer: gst::Buffer,
-    ) -> Option<gst::Buffer>;
-
-    fn parent_finish_buffer(&self, buffer: gst::Buffer)
-        -> Result<gst::FlowSuccess, gst::FlowError>;
-
-    #[cfg(any(feature = "v1_18", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
-    fn parent_finish_buffer_list(
-        &self,
-        buffer_list: gst::BufferList,
-    ) -> Result<gst::FlowSuccess, gst::FlowError>;
-
-    fn parent_sink_event(&self, aggregator_pad: &AggregatorPad, event: gst::Event) -> bool;
-
-    #[cfg(any(feature = "v1_18", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
-    fn parent_sink_event_pre_queue(
-        &self,
-        aggregator_pad: &AggregatorPad,
-        event: gst::Event,
-    ) -> Result<gst::FlowSuccess, gst::FlowError>;
-
-    fn parent_sink_query(&self, aggregator_pad: &AggregatorPad, query: &mut gst::QueryRef) -> bool;
-
-    #[cfg(any(feature = "v1_18", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
-    fn parent_sink_query_pre_queue(
-        &self,
-        aggregator_pad: &AggregatorPad,
-        query: &mut gst::QueryRef,
-    ) -> bool;
-
-    fn parent_src_event(&self, event: gst::Event) -> bool;
-
-    fn parent_src_query(&self, query: &mut gst::QueryRef) -> bool;
-
-    fn parent_src_activate(
-        &self,
-        mode: gst::PadMode,
-        active: bool,
-    ) -> Result<(), gst::LoggableError>;
-
-    fn parent_aggregate(&self, timeout: bool) -> Result<gst::FlowSuccess, gst::FlowError>;
-
-    fn parent_start(&self) -> Result<(), gst::ErrorMessage>;
-
-    fn parent_stop(&self) -> Result<(), gst::ErrorMessage>;
-
-    fn parent_next_time(&self) -> Option<gst::ClockTime>;
-
-    fn parent_create_new_pad(
-        &self,
-        templ: &gst::PadTemplate,
-        req_name: Option<&str>,
-        caps: Option<&gst::Caps>,
-    ) -> Option<AggregatorPad>;
-
-    fn parent_update_src_caps(&self, caps: &gst::Caps) -> Result<gst::Caps, gst::FlowError>;
-
-    fn parent_fixate_src_caps(&self, caps: gst::Caps) -> gst::Caps;
-
-    fn parent_negotiated_src_caps(&self, caps: &gst::Caps) -> Result<(), gst::LoggableError>;
-
-    fn parent_propose_allocation(
-        &self,
-        pad: &AggregatorPad,
-        decide_query: Option<&gst::query::Allocation>,
-        query: &mut gst::query::Allocation,
-    ) -> Result<(), gst::LoggableError>;
-
-    fn parent_decide_allocation(
-        &self,
-        query: &mut gst::query::Allocation,
-    ) -> Result<(), gst::LoggableError>;
-
-    #[cfg(any(feature = "v1_18", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
-    fn parent_negotiate(&self) -> bool;
-
-    #[cfg(any(feature = "v1_18", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
-    fn parent_peek_next_sample(&self, pad: &AggregatorPad) -> Option<gst::Sample>;
+mod sealed {
+    pub trait Sealed {}
+    impl<T: super::AggregatorImplExt> Sealed for T {}
 }
 
-impl<T: AggregatorImpl> AggregatorImplExt for T {
+pub trait AggregatorImplExt: sealed::Sealed + ObjectSubclass {
     fn parent_flush(&self) -> Result<gst::FlowSuccess, gst::FlowError> {
         unsafe {
             let data = Self::type_data();
@@ -280,8 +194,8 @@ impl<T: AggregatorImpl> AggregatorImplExt for T {
         }
     }
 
-    #[cfg(any(feature = "v1_18", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
+    #[cfg(feature = "v1_18")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_18")))]
     fn parent_finish_buffer_list(
         &self,
         buffer_list: gst::BufferList,
@@ -314,8 +228,8 @@ impl<T: AggregatorImpl> AggregatorImplExt for T {
         }
     }
 
-    #[cfg(any(feature = "v1_18", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
+    #[cfg(feature = "v1_18")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_18")))]
     fn parent_sink_event_pre_queue(
         &self,
         aggregator_pad: &AggregatorPad,
@@ -350,8 +264,8 @@ impl<T: AggregatorImpl> AggregatorImplExt for T {
         }
     }
 
-    #[cfg(any(feature = "v1_18", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
+    #[cfg(feature = "v1_18")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_18")))]
     fn parent_sink_query_pre_queue(
         &self,
         aggregator_pad: &AggregatorPad,
@@ -630,8 +544,8 @@ impl<T: AggregatorImpl> AggregatorImplExt for T {
         }
     }
 
-    #[cfg(any(feature = "v1_18", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
+    #[cfg(feature = "v1_18")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_18")))]
     fn parent_negotiate(&self) -> bool {
         unsafe {
             let data = Self::type_data();
@@ -649,8 +563,8 @@ impl<T: AggregatorImpl> AggregatorImplExt for T {
         }
     }
 
-    #[cfg(any(feature = "v1_18", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
+    #[cfg(feature = "v1_18")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "v1_18")))]
     fn parent_peek_next_sample(&self, pad: &AggregatorPad) -> Option<gst::Sample> {
         unsafe {
             let data = Self::type_data();
@@ -668,6 +582,8 @@ impl<T: AggregatorImpl> AggregatorImplExt for T {
     }
 }
 
+impl<T: AggregatorImpl> AggregatorImplExt for T {}
+
 unsafe impl<T: AggregatorImpl> IsSubclassable<T> for Aggregator {
     fn class_init(klass: &mut glib::Class<Self>) {
         Self::parent_class_init::<T>(klass);
@@ -690,7 +606,7 @@ unsafe impl<T: AggregatorImpl> IsSubclassable<T> for Aggregator {
         klass.negotiated_src_caps = Some(aggregator_negotiated_src_caps::<T>);
         klass.propose_allocation = Some(aggregator_propose_allocation::<T>);
         klass.decide_allocation = Some(aggregator_decide_allocation::<T>);
-        #[cfg(any(feature = "v1_18", feature = "dox"))]
+        #[cfg(feature = "v1_18")]
         {
             klass.sink_event_pre_queue = Some(aggregator_sink_event_pre_queue::<T>);
             klass.sink_query_pre_queue = Some(aggregator_sink_query_pre_queue::<T>);
@@ -738,8 +654,8 @@ unsafe extern "C" fn aggregator_finish_buffer<T: AggregatorImpl>(
     .into_glib()
 }
 
-#[cfg(any(feature = "v1_18", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
+#[cfg(feature = "v1_18")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_18")))]
 unsafe extern "C" fn aggregator_finish_buffer_list<T: AggregatorImpl>(
     ptr: *mut ffi::GstAggregator,
     buffer_list: *mut gst::ffi::GstBufferList,
@@ -767,8 +683,8 @@ unsafe extern "C" fn aggregator_sink_event<T: AggregatorImpl>(
     .into_glib()
 }
 
-#[cfg(any(feature = "v1_18", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
+#[cfg(feature = "v1_18")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_18")))]
 unsafe extern "C" fn aggregator_sink_event_pre_queue<T: AggregatorImpl>(
     ptr: *mut ffi::GstAggregator,
     aggregator_pad: *mut ffi::GstAggregatorPad,
@@ -801,8 +717,8 @@ unsafe extern "C" fn aggregator_sink_query<T: AggregatorImpl>(
     .into_glib()
 }
 
-#[cfg(any(feature = "v1_18", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
+#[cfg(feature = "v1_18")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_18")))]
 unsafe extern "C" fn aggregator_sink_query_pre_queue<T: AggregatorImpl>(
     ptr: *mut ffi::GstAggregator,
     aggregator_pad: *mut ffi::GstAggregatorPad,
@@ -1054,8 +970,8 @@ unsafe extern "C" fn aggregator_decide_allocation<T: AggregatorImpl>(
     .into_glib()
 }
 
-#[cfg(any(feature = "v1_18", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
+#[cfg(feature = "v1_18")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_18")))]
 unsafe extern "C" fn aggregator_negotiate<T: AggregatorImpl>(
     ptr: *mut ffi::GstAggregator,
 ) -> glib::ffi::gboolean {
@@ -1065,8 +981,8 @@ unsafe extern "C" fn aggregator_negotiate<T: AggregatorImpl>(
     gst::panic_to_error!(imp, false, { imp.negotiate() }).into_glib()
 }
 
-#[cfg(any(feature = "v1_18", feature = "dox"))]
-#[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
+#[cfg(feature = "v1_18")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_18")))]
 unsafe extern "C" fn aggregator_peek_next_sample<T: AggregatorImpl>(
     ptr: *mut ffi::GstAggregator,
     pad: *mut ffi::GstAggregatorPad,
